@@ -18,7 +18,7 @@
 # SUCH DAMAGE.
 #
 # Author: Andrew Milton <akm@theinternet.com.au>
-# $Id: exUserFolder.py,v 1.89 2003/07/05 21:53:09 akm Exp $
+# $Id: exUserFolder.py,v 1.93 2004/11/10 14:15:33 akm Exp $
 
 ##############################################################################
 #
@@ -98,6 +98,7 @@ from OFS.CopySupport import CopyContainer
 from base64 import decodestring, encodestring
 from urllib import quote, unquote
 
+from Acquisition import aq_base
 from AccessControl import ClassSecurityInfo
 from AccessControl.Role import RoleManager
 from AccessControl.User import BasicUser, BasicUserFolder, readUserAccessFile
@@ -115,6 +116,9 @@ from UserCache.UserCache import GlobalUserCache, GlobalNegativeUserCache, Global
 
 from LoginRequiredMessages import LoginRequiredMessages
 
+from AccessControl import Unauthorized
+
+
 
 # If there is no NUG Product just define a dummy class
 try:
@@ -143,15 +147,16 @@ def manage_addexUserFolder(self, authId, propId, memberId,
 						   cookie_mode=0, session_length=0,
 						   not_session_length=0,
 						   sessionTracking=None, idleTimeout=None,
-						   REQUEST={}, groupId=None):
+						   REQUEST={}, groupId=None, cryptoId=None):
 	""" """
 	if hasattr(self.aq_base, 'acl_users'):
 		return Globals.MessageDialog(self,REQUEST,
 			title  ='Item Exists',
 			message='This object already contains a User Folder',
 			action ='%s/manage_main' % REQUEST['URL1'])
-	ob=exUserFolder(authId, propId, memberId, groupId, cookie_mode,
-			session_length, sessionTracking, idleTimeout, not_session_length)
+	ob=exUserFolder(authId, propId, memberId, groupId, cryptoId, cookie_mode,
+					session_length, sessionTracking, idleTimeout,
+					not_session_length)
 			
 	self._setObject('acl_users', ob, None, None, 0)
 	self.__allow_groups__=self.acl_users
@@ -186,6 +191,7 @@ class exUserFolder(Folder,BasicUserFolder,BasicGroupFolderMixin,
 	__allow_access_to_unprotected_subobjects__=1
 	authSources={}
 	propSources={}
+	cryptoSources={}
 	membershipSources={}
 	groupSources={}
 
@@ -268,6 +274,7 @@ class exUserFolder(Folder,BasicUserFolder,BasicGroupFolderMixin,
 	manage_contents=Folder.manage_main
 	manage_showCacheData=DTMLFile('dtml/manage_showCacheData', globals())
 
+	# This is going away soon...
 	docLoginRedirect=DTMLFile('dtml/docLoginRedirect', globals())	
 
 	# Stupid crap
@@ -318,9 +325,9 @@ class exUserFolder(Folder,BasicUserFolder,BasicGroupFolderMixin,
 	# sessionTracking=None # Or session tracking.
 	# idleTimeout=0
 	
-	def __init__(self, authId, propId, memberId, groupId, cookie_mode=0,
-				 session_length=0, sessionTracking=None, idleTimeout=0,
-				 not_session_length=0):
+	def __init__(self, authId, propId, memberId, groupId, cryptoId,
+				 cookie_mode=0, session_length=0, sessionTracking=None,
+				 idleTimeout=0, not_session_length=0):
 		self.cookie_mode=cookie_mode
 		self.sessionLength=session_length
 		self.notSessionLength=not_session_length
@@ -350,6 +357,11 @@ class exUserFolder(Folder,BasicUserFolder,BasicGroupFolderMixin,
 		else:
 			self.manage_addGroupSource=None
 			self.currentGroupsSource=None
+
+		if cryptoId:
+			self.cryptoId = cryptoId
+		else:
+			self.cryptoId = 'Crypt'
 			
 	def __setstate__(self, state):
 		Persistent.__setstate__(self, state)
@@ -469,14 +481,14 @@ class exUserFolder(Folder,BasicUserFolder,BasicGroupFolderMixin,
 			x = XUFUserCache.createCache(pp, self.sessionLength)
 		x.addToCache(username, password, user)
 
-	def cache_getUser(self, username, password):
+	def cache_getUser(self, username, password, checkpassword=1):
 		if not self.sessionLength:
 			return None
 		pp = string.join(self.getPhysicalPath(), '/')
 		x = XUFUserCache.getCache(pp)
 		if not x:
 			return None
-		u = x.getUser(self, username, password)
+		u = x.getUser(self, username, password, checkpassword)
 		if u is not None:
 			u = u.__of__(self)
 		return u
@@ -767,6 +779,9 @@ class exUserFolder(Folder,BasicUserFolder,BasicGroupFolderMixin,
 	
 	def getUser(self, name):
 		"""Return the named user object or None if no such user exists"""
+		user = self.cache_getUser(name, '', 0)
+		if user:
+			return user
 		try:
 			items=self.listOneUser(name)
 		except:
@@ -894,7 +909,7 @@ class exUserFolder(Folder,BasicUserFolder,BasicGroupFolderMixin,
 				else:
 					return None
 			else:
-				self.challenge(request, response, 'unauthorized', auth)
+				self.challenge(request, response, None, auth)
 				return None
 	
 	def authenticate(self, name, password, request):
@@ -931,8 +946,9 @@ class exUserFolder(Folder,BasicUserFolder,BasicGroupFolderMixin,
 			if reason_code == 'login_failed':
 				response.expireCookie('__ac', path='/')
 				response.expireCookie('__aca', path='/')
-			request.set('authFailedCode', reason_code)
-			raise 'LoginRequired', self.docLoginRedirect(self, request)
+			if reason_code:
+				request.set('authFailedCode', reason_code)
+			raise 'LoginRequired', self.docLogin(self, request)
 		else:
 			zLOG.LOG('exUserFolder', zLOG.DEBUG, 'not raising LoginRequired for %s' % reason_code)
 
@@ -1004,7 +1020,7 @@ class exUserFolder(Folder,BasicUserFolder,BasicGroupFolderMixin,
 			c=decodestring(c)
 		except:
 			response.expireCookie('__ac', path='/')
-			raise 'LoginRequired', self.docLoginRedirect(self, request)
+			raise 'LoginRequired', self.docLogin(self, request)
 		
 		name,password=tuple(split(c, ':', 1))
 		return name, password
@@ -1018,7 +1034,7 @@ class exUserFolder(Folder,BasicUserFolder,BasicGroupFolderMixin,
 			response.expireCookie('__aca', path='/')
 			response.expireCookie('__ac', path='/')	# Precaution
 			response.flush()
-			raise 'LoginRequired', self.docLoginRedirect(self, request)
+			raise 'LoginRequired', self.docLogin(self, request)
 
 		u = self.cache_getCookieCacheUser(c)
 		if u:
@@ -1027,7 +1043,7 @@ class exUserFolder(Folder,BasicUserFolder,BasicGroupFolderMixin,
 		response.expireCookie('__aca', path='/')
 		response.expireCookie('__ac', path='/')	# Precaution
 		response.flush()
-		raise 'LoginRequired', self.docLoginRedirect(self, request)
+		raise 'LoginRequired', self.docLogin(self, request)
 
 	def setBasicCookie(self, name, password, request, response):
 		token='%s:%s' % (name, password)
@@ -1133,7 +1149,12 @@ class exUserFolder(Folder,BasicUserFolder,BasicGroupFolderMixin,
 		return self.currentAuthSource.listOneUser(username)
 
 	def cryptPassword(self, username, password):
-		return self.currentAuthSource.cryptPassword(username, password)
+		if hasattr(aq_base(self.currentAuthSource), 'cryptPassword'):
+			return self.currentAuthSource.cryptPassword(username, password)
+
+		if hasattr(self, 'cryptoId'):
+			return self.cryptoSources[self.cryptoId].plugin(self, username, password)
+		return self.cryptoSources['Crypt'].plugin(self, username, password)
 
 	def PropertyEditor(self):
 		""" """
@@ -1226,6 +1247,9 @@ class exUserFolder(Folder,BasicUserFolder,BasicGroupFolderMixin,
 		if self.currentGroupSource:
 			return self.currentGroupSource.delGroupsFromUser(groupnames, username)
 
+	# We lie.
+	def hasUsers(self):
+		return 1
 
 
 def doAuthSourceForm(self,authId):
@@ -1280,6 +1304,15 @@ def getGroupSources(self):
 			)
 	return l
 
+def getCryptoSources(self):
+	""" Doc String """
+	l = []
+	for o in exUserFolder.cryptoSources.keys():
+		l.append(
+			exUserFolder.cryptoSources[o]
+			)
+	return l
+
 def MailHostIDs(self):
     """Find SQL database connections in the current folder and above
 
